// this file is part of
// markonvert - Fortran unformatted record marker converter
//
// Copyright (C) 2005-2007  Francesco Poli
//
// This work is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 2,
// as published by the Free Software Foundation.
// 
// This work is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this work; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#ifndef MK_CONVERTER_H
#define MK_CONVERTER_H

#include <iostream>
#include "myassert.h"

typedef int int32;

namespace mk
{
    // Exceptions
    class Ex_int_not_4   { }; // int32 is not 4 byte long
    class Ex_chr_not_1   { }; // char  is not 1 byte long
    class Ex_neg_length  { }; // found negative record length
    class Ex_long_record { }; // found record longer than (pow(2,31) - 1) byte
    class Ex_marks_diff  { }; // end marker differs from start marker

    // Base class for a general converter
    class Converter
      {
        public:
            Converter()
                : record_len(0)
                  {
                    Assert<Ex_int_not_4>(sizeof(int32) == 4);
                    Assert<Ex_chr_not_1>(sizeof(char ) == 1);
                    std::cin.exceptions(std::ios::goodbit);
                  }

            void go();  // perform the conversion

            virtual ~Converter() { }

        protected:
            virtual void read_start_mark() = 0;
            virtual void read_end_mark() = 0;
            virtual void write_mark() = 0;

            void copy_record();


            int32 record_len;
      };
}

#endif // not MK_CONVERTER_H
